// AT90S8535
#include <avr/io.h>
#include <avr/interrupt.h>	//    sei()
#include <avr/signal.h>		//    INTERRUPT
#include <avr/delay.h>		//    _delay_loop_2
#include <string.h>			//    sprintf

/*         ,      -: 
PB0 - ; 1 - RS; PB2 - RD */
#define RD_1 PORTB |= 4	//PORTB = PORTB | 0b100 - RD = 1 
#define RS_1 PORTB |= 2	//PORTB = PORTB | 0b010 - RS = 1 
#define E_1  PORTB |= 1	//PORTB = PORTB | 0b001 - E = 1
#define RD_0 PORTB &= 3	//PORTB = PORTB & 0b011 - RD = 0 
#define RS_0 PORTB &= 5	//PORTB = PORTB & 0b101 - RS = 0 
#define E_0  PORTB &= 6	//PORTB = PORTB & 0b110 - E = 0
 

const unsigned char addLUT[4] = {0x80, 0xC0, 0x94, 0xD4};//      
									//-,       4-   
unsigned char LCD_Address, LCD_Line;	//        -
char buffer[15];						//   ,   -

  
void WriteNibble(unsigned char data)	//   -  
{
  RD_0;						// 
  E_1;						// 
  PORTC = (data & 0x0F);	// 
  E_0;						// 
  RD_1;						// 
  _delay_loop_2(3000);		// 3 ,  -       

}

void WriteByte(unsigned char data)	//   -  
{
  RD_0;						// 
  E_1;						// 
  PORTC = (data >> 4);		//   
  E_0;						// 
  E_1;						// 
  PORTC = (data & 0xF);		//   
  E_0;						// 
  RD_1;						// 
  _delay_loop_2(3000);		// 3 ,  -      

}

void GoToLine(char LineNum)	//        LineNum
{
  RS_0;								// 
  LCD_Address = addLUT[LineNum-1];	//   
  WriteByte(LCD_Address);			//    -
  RS_1;								//  
  LCD_Address = 0;					// 
  LCD_Line = LineNum;				//  
}

void SetLCDPosition(char row, char col)	//    col   row 
{
  RS_0;									// 
  LCD_Address = addLUT[row-1] + col;	//   
  WriteByte(LCD_Address);				//  - 
  RS_1;									//  
  LCD_Line = row;						//  
}

void ShowChar(unsigned char c)	//       
{
  RS_1;						// 
  WriteByte(c);				// 
  LCD_Address++;			//   1
  switch (LCD_Address)		//,   
  {							//    
    case 20: GoToLine(2); break;
    case 40: GoToLine(3); break;
	case 60: GoToLine(4); break;
	case 80: GoToLine(1); break;
  }
}

void ShowStr(unsigned char *s)	//    
{
  while (*s != 0) ShowChar(*s++);
}

void InitLCD(void)	//  - 
{
  unsigned char title1[] = "Temperature: ";//  
  unsigned char title2[] = "Motor velocity: ";//   -
  RD_1;//   
  E_0;
  RS_0;//  
  _delay_loop_2(50000);				//  50 ,  -     
  WriteNibble(0x33);				// , 
  WriteNibble(0x33);				// -    
  WriteNibble(0x33);				// 4-   (. . ) 
  WriteNibble(0x22);
  WriteByte(0x28);					//   ( 5x7) 
  WriteByte(0x01);
  WriteByte(0x10);					//    
  WriteByte(0x06);					//,     
  WriteByte(0x0C);					// ,   
  for (char i=0x40; i<0x5F; i++)	//  
  {									//
    _delay_loop_2(10000);
	RS_0;
	WriteByte(i);
	_delay_loop_2(10000);
	ShowChar(0);
  }
  RS_1;								//    
  SetLCDPosition(1, 4);				//     
  ShowStr(title1);					// "Temperature:" 
  SetLCDPosition(3, 2);				//     
  ShowStr(title2);					// "Motor velocity:"
}

ISR(ADC_vect)	//   - 
{
  unsigned char CurrentTemp, CurrentVel;
  CurrentTemp = ((long)101 * (long)ADCW) / (long) 1023 - (long)40;	//   
																		//   ADCW		
  CurrentVel = (CurrentTemp - 30) * 1022 / 30;	//      OCR1A
  sprintf(buffer, "%02d C", CurrentTemp);	//  
  SetLCDPosition(2, 8);						//    -
  ShowStr(buffer);
  sprintf(buffer, "%04d", CurrentVel);	//   
  SetLCDPosition(4, 8);					//          -
  ShowStr(buffer);
  OCR1A = CurrentVel;//     
}

void main (void)
{
  DDRA = 0;				//   - 
  DDRB = 0x07;			// 0-3   -  ( )
  DDRC = 0x0F;			// 0-4   -  ( )
  DDRD = _BV(PD5);		// 5  D -  1   
  TIMSK = _BV(TOIE1);	//    /1  10-, 
						//  ,    
  TCCR1A = _BV(PWM10) | _BV(PWM11) | _BV(COM1A1);
  TCCR1B = _BV(CS10);	//   
  TCNT1 = 0;			//   0
  OCR1A = 0;			//    
  ACSR = _BV(ACD);		//  
  ADMUX = 3;		//   3,  AREF -     
  /*     ,    64,    
    8     125  
  (       (50 - 200 ) */ 

  ADCSR = _BV(ADEN) | _BV(ADSC) | _BV(ADFR) | _BV(ADIE) | _BV(ADPS2) | _BV(ADPS1);
  InitLCD();	// -
  sei();		//  
  while(1) ;	// 
}
